/**************************************************************************
	Header File for Win32 compatibilty

	Written by Atif Aziz, Sep 1993
	Modifed by Daniel Doguet

	This file helps Win16 applications to be more source-compatible
	with Win32. It defines certain values that translate to the most
	appropriate values depending on the target environment.
**************************************************************************/

#ifndef _WINXEXT_H_
#define _WINXEXT_H_

#ifndef __WINDOWS_H
#include <windows.h>
#endif

/* Macro to define pointer version of a given typedef */

#define TYPEDEF_POINTERS(base, type)            \
typedef base *                  P##type;        \
typedef const base *            PC##type;       \
typedef base NEAR *             NP##type;       \
typedef const base NEAR *       NPC##type;      \
typedef base FAR *              LP##type;       \
typedef const base FAR *        LPC##type;

/*-------------------------------------------------------------------------
	 WIN16 and WIN32 Compatibility definitions 
  -------------------------------------------------------------------------*/

#ifdef WIN32

  /* Portable resource character type */
  typedef WCHAR RCHAR;

  /* Macro returning the length of a resource string.  */
  #define _rcslen(lpcrsz)	lstrlenW(lpcrsz)

	/* WIN32 does not need to modify callback entry point */
	#define _EXPORT

  /* Missing lstrcpyn() from Win32 API */

  #include <tchar.h>
  #include <wchar.h>

	#ifndef NOGDI

	#pragma pack(1)

	/* 16 bit structure */
	typedef struct tagLOGFONT16
	{
	    short   lfHeight;
	    short   lfWidth;
	    short   lfEscapement;
	    short   lfOrientation;
	    short   lfWeight;
	    BYTE    lfItalic;
	    BYTE    lfUnderline;
	    BYTE    lfStrikeOut;
	    BYTE    lfCharSet;
	    BYTE    lfOutPrecision;
	    BYTE    lfClipPrecision;
	    BYTE    lfQuality;
	    BYTE    lfPitchAndFamily;
	    char    lfFaceName[32];
	} LOGFONT16, *PLOGFONT16, NEAR* NPLOGFONT16, FAR* LPLOGFONT16;

	/* Restore compiler default  */
	#pragma pack()

	/*
	inline void LogFont16ToLogFont32(PLOGFONT16 plf16, LPLOGFONT plf32)
	{
	  plf32->lfHeight =	plf16->lfHeight;        
	  plf32->lfWidth = plf16->lfWidth;         
	  plf32->lfEscapement =	plf16->lfEscapement;    
	  plf32->lfOrientation = plf16->lfOrientation;   
	  plf32->lfWeight = plf16->lfWeight;        
	  plf32->lfItalic = plf16->lfItalic;        
	  plf32->lfUnderline = plf16->lfUnderline;     
	  plf32->lfStrikeOut = plf16->lfStrikeOut;     
	  plf32->lfCharSet = plf16->lfCharSet;       
	  plf32->lfOutPrecision = plf16->lfOutPrecision;  
	  plf32->lfClipPrecision = plf16->lfClipPrecision; 
	  plf32->lfQuality = plf16->lfQuality;       
	  plf32->lfPitchAndFamily = plf16->lfPitchAndFamily;
	#ifdef UNICODE
		MultiByteToWideChar(CP_ACP, 0, plf16->lfFaceName, -1, plf32->lfFaceName, sizeof(plf32->lfFaceName));
	#else
		lstrcpy(plf32->lfFaceName, plf16->lfFaceName);
	#endif
	}
	*/

	#endif /* NOGDI */

	#define HUGE

#else	/* WIN32 */

  /* Portable basic character unit not included in Win16 header files
     but is defined here. */

  typedef char  TCHAR;

  TYPEDEF_POINTERS(TCHAR, TSTR)
  TYPEDEF_POINTERS(TCHAR, TCH)

  /* Macro returning the length of a resource string. */
  #define _rcslen(lpcsz)	lstrlen(lpcsz)

	#ifndef _EXPORT 
	#define _EXPORT _export
	#endif

  /* The following macro is to be used for string and character literals
     appearing in the code. It is already defined in the Win32 SDK
     header file. */
  #ifndef TEXT
  #define TEXT(text)            text
	#endif
	
  /* Portable resource character type  */
  typedef char  RCHAR;

	#define LOGFONT16 LOGFONT

	#define HUGE _huge

#endif	/* WIN32 */

/*-------------------------------------------------------------------------
  Definitions common to WIN16 and WIN32 
  ------------------------------------------------------------------------- */

/* Define string versions of RCHAR (PRSTR, LPRSTR, etc.) */
TYPEDEF_POINTERS(RCHAR, RSTR)

	/* Miscellaneous type definitions */

typedef HWND NEAR *	NPHWND;

	/* Other useful macros */

#define ARRAY_COUNT(a)		(sizeof(a) / sizeof((a)[0]))

#endif  /* _WINXEXT_H_ */
